package com.xy.utils;

import cn.hutool.core.bean.BeanUtil;
import com.xy.collections.list.JArrayList;
import com.xy.collections.map.JMap;
import com.xy.dto.MsgConfigDto;
import com.xy.dto.MsgConfigTestDto;
import com.xy.service.MsgSendApiService;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

public class MsgUtils {

    /**
     * 发送消息
     *
     * @param configId 消息配置id
     * @param func     消息内容集合
     */
    public static void sendMsg(Long configId, FunctionUtils.NoParamsResult<List<MsgInfo>> func) {
        List<MsgInfo> msgInfos = func.run();
        sendMsg(configId, msgInfos);
    }

    /**
     * 发送消息
     *
     * @param configId 消息配置id
     * @param msgInfos 消息内容集合
     */
    public static void sendMsg(Long configId, List<MsgInfo> msgInfos) {
        if (!Emptys.check(msgInfos)) {
            return;
        }
        MsgSendApiService msgSendApiService = SpringBeanUtils.getBean(MsgSendApiService.class);
        JMap<Integer, List<MsgInfo>> msgInfosJMaps = new JArrayList<>(msgInfos).toMap(MsgInfo::getChannelType).group();
        //获取消息配置
        MsgConfigDto.Vo msgConfig = R.feignCheckData(msgSendApiService.getMsgConfig(new MsgConfigDto.Vo().setId(configId)));
        if (!Emptys.check(msgConfig)) {
            return;
        }
        //获取消息配置业务参数
        List<MsgConfigTestDto.BizParam> bizParams = R.feignCheckData(msgSendApiService.getBizParamByMsgConfig(new MsgConfigTestDto.MsgConfig().setConfigId(configId)));
        if (!Emptys.check(bizParams)) {
            return;
        }
        List<MsgConfigTestDto.BizData> bizDataList = BeanUtil.copyToList(bizParams, MsgConfigTestDto.BizData.class);
        List<MsgConfigTestDto.BizData> sendList = new ArrayList<>();
        bizDataList.forEach(bizData -> {
            List<MsgInfo> msgInfosList = msgInfosJMaps.get(Integer.valueOf(bizData.getChannelType()).intValue());
            if (!Emptys.check(msgInfosList)) {
                return;
            }
            msgInfosList.forEach(msgInfo -> {
                bizData.setReceivers(new HashSet<>(msgInfo.getMpOpenIds()))
                        .setTemplateParams(msgInfo.getParams());
                sendList.add(bizData);
            });
        });
        //消息推送
        msgSendApiService.sendByMsgConfig(new MsgConfigTestDto.SendByMsgConfig().setConfigId(configId).setBizDataList(sendList));
    }

    @Data
    @Accessors(chain = true)
    public static class MsgInfo {

        /**
         * 发送渠道类型，枚举 ChannelType
         */
        private Integer channelType;

        /**
         * 接收人
         */
        private List<String> mpOpenIds;

        /**
         * 消息内容
         */
        private Map<String, Object> params;
    }
}
